package file;

#//////////////////////////////////////////////////////////////////////////////#
#
#　ファイル操作関連の処理を行うライブラリ
#
#//////////////////////////////////////////////////////////////////////////////#
use strict;

#******************************************************************************#
#
#　ファイル内容取得処理(複数行)
#
#　dataFile : ファイル名
#　return   : ファイル内容一覧 or -1( 処理失敗 )
#
#******************************************************************************#
sub getFileDataList{

	#引数を取得
	my( $fileName ) = @_;

	#ローカル変数の宣言
	my( @dataList );

	#書き込みデータ管理用ファイルオープン : 読み込みモード
	open( IN, "$fileName" ) || die return -1;

	#書き込み内容全て取得
	@dataList = <IN>;

	#ファイルクローズ
	close IN;

	#配列返却
	return @dataList;
}

#******************************************************************************#
#
#　ファイル内容取得処理(一行)
#
#　dataFile : ファイル名
#　return   : ファイル内容 or -1( 処理失敗 )
#
#******************************************************************************#
sub getFileData{

	#引数を取得
	my( $fileName ) = @_;

	#ファイル読み込み
	open( IN, "$fileName" ) || die return -1;

	#データ取得
	my( $data ) = <IN>;

	#ファイルクローズ
	close( IN );

	#データ返却
	return $data;
}

#******************************************************************************#
#
#　ファイル内容更新処理(複数行)
#
#　dataList : ファイル内容を更新するデータ一覧
#　fileName : ファイル名
#　return   : 1( 処理成功 ) or -1( 処理失敗 )
#
#******************************************************************************#
sub updateFileDataList{

	#引数を取得
	my( $dataList, $fileName ) = @_;

	#書き込みデータ管理用ファイルオープン : 書き込みモード
	open( OUT, "> $fileName" ) || die return -1;

	#書き込み情報全て書き込み
	print OUT @$dataList;

	#ファイルクローズ
	close OUT;

	#正常値返却
	return 1;
}

#******************************************************************************#
#
#　ファイル内容更新処理(一行)
#
#　dataList : ファイル内容を更新するデータ
#　fileName : ファイル名
#　return   : 1( 処理成功 ) or -1( 処理失敗 )
#
#******************************************************************************#
sub updateFileData{

	#引数を取得
	my( $data, $fileName ) = @_;

	#書き込みデータ管理用ファイルオープン : 書き込みモード
	open( OUT, "> $fileName" ) || die return -1;

	#書き込み情報全て書き込み
	print OUT $data;

	#ファイルクローズ
	close OUT;

	#正常値返却
	return 1;
}

#******************************************************************************#
#
#　データ追加書き込み処理
#
#　moreData : ファイルに書き込むデータ
#　fileName : ファイル名
#　return   : 1( 処理成功 ) or -1( 処理失敗 )
#
#******************************************************************************#
sub plusFileData{

	#引数を取得
	my( $moreData, $fileName ) = @_;

	#追加書き込みモード
	open( OUT, ">> $fileName" ) || die return -1;

	#書き込み
	print OUT $moreData;

	#ファイルクローズ
	close OUT;

	#正常値返却
	return 1;
}

#******************************************************************************#
#
#　ロック用ファイルをロックし、他プロセスとの衝突を避けるための処理
#
#　fileName : ファイル名
#
#******************************************************************************#
sub lockExe{

	#引数を取得
	my( $fileName ) = @_;

	#ロック用ファイル ロック
	open( DANGO, "> $fileName" );
	flock( DANGO, 2 );
}

#******************************************************************************#
#
#　ロック用ファイルをアンロックし、他プロセスを処理可能にするための処理
#
#******************************************************************************#
sub unlockExe{

	#ロック用ファイル アンロック
	flock( DANGO, 8 );
	close( DANGO );
}

1;
