#!/usr/local/bin/perl

#//////////////////////////////////////////////////////////////////////////////#
#
#　ランキング登録処理を行うCGIファイル
#
#------------------------------------------------------------------------------#
#　必要パラメータ
#
#　　mode : 処理モード
#　　rank : ランキング記録最大数 (mode2のみ必要)
#　　data : 登録データ           (mode2のみ必要) (csv形式で渡す)
#
#　mode param 説明
#　　mode=1 : ランキングデータ表示
#  　mode=2 : データ登録 ＆ ランキングデータ表示
#	 mode=3 : ユニークID取得
#
#//////////////////////////////////////////////////////////////////////////////#
use strict;
use CGI qw(:standard);
require 'lib/file.pl';

#//////////////////////////////////////////////////////////////////////////////#
#
#　変数・定数定義
#
#//////////////////////////////////////////////////////////////////////////////#

#******************************************************************************#
#
#　処理用
#
#******************************************************************************#
#ランキング最大数
our $DATA_MAX_NUM = 0;

#******************************************************************************#
#
#　ファイル関連
#
#******************************************************************************#
#datファイル配置ディレクトリ名
our $DAT_DIR = "dat/";

#ランキング管理用ファイル名
#
# カラム内容 : 0～1までは定型 2以降は自由
# 　0 : ポイント(並び替え対象)
# 　1 : ID
#
# 　2 : 名前
#
our $RANK_FILE = $DAT_DIR . "rank.dat";

#ロック用ファイル名
our $LOCK_FILE = $DAT_DIR . "lock.dat";

#ID番号管理用ファイル名
our $ID_FILE = $DAT_DIR . "id.dat";

#//////////////////////////////////////////////////////////////////////////////#
#
#　処理開始
#
#//////////////////////////////////////////////////////////////////////////////#
&mainExe();

#******************************************************************************#
#
#　メイン処理開始
#
#******************************************************************************#
sub mainExe{
	
	#ヘッダ表示
	print "Content-type: text/plane\n\n";
	#print "Content-type: text/html\n\n";

	#POSTでリクエストが送信されなかった場合 : エラー
	if( $ENV{'REQUEST_METHOD'} ne 'POST' ){ return; }

	#ロック用ファイル ロック
	&file::lockExe( $LOCK_FILE );

	#処理判断パラメータの取得
	my $mode = param( 'mode' );

	#ランキング登録件数の取得
	$DATA_MAX_NUM = param( 'rank' );

	#処理判断
	if   ( $mode == 1 ){ &dispExe  (); }
	elsif( $mode == 2 ){ &registExe();
			     		 &dispExe  (); }
	elsif( $mode == 3 ){ &getUniqID(); }

	#ロック用ファイル アンロック
	&file::unlockExe();
}

#******************************************************************************#
#
#　データ表示処理 : mode1 or mode2
#
#******************************************************************************#
sub dispExe{

	#表示用一覧取得
	my @dataList = &file::getFileDataList( $RANK_FILE );

	#表示開始
	my $i = 0;
	foreach my $dispData ( @dataList ){

		chop $dispData;
		print "&data$i=$dispData";
		$i++;
	}
	print "&";
}

#******************************************************************************#
#
#　データ登録処理 : mode2
#
#******************************************************************************#
sub registExe{

	#登録内容取得
	my $data = param( 'data' );	

	#データ管理用一覧取得
	my @dataList = &file::getFileDataList( $RANK_FILE );

	#ランクインしていない場合 : 処理終了
	if( &checkRankIn( $data, \@dataList ) == 0 ){ print "&rank=0&"; return; }

	#同一IDデータ調査
	&checkSameID( $data, \@dataList );

	#ランクソート処理
	@dataList = sort{ (split(/\,/,$b))[0] <=> (split(/\,/,$a))[0] }@dataList;

	#ランク管理ファイル更新
	&file::updateFileDataList( \@dataList, $RANK_FILE );

	#自分のランキング番号を取得
	my $rankNum = &getRankNum( $data, \@dataList );

	#ランクを表示
	print "&rank=$rankNum&";
}

#******************************************************************************#
#
#　ユニークID取得処理開始 : mode3
#
#******************************************************************************#
sub getUniqID{

	#カウンタファイル読み込み
	my( $id ) = &file::getFileData( $ID_FILE );

	#カウントアップ
	$id++;

	#カウントデータ更新
	&file::updateFileData( $id, $ID_FILE );

	#ID表示
	print "&id=$id&";
}

#******************************************************************************#
#
#　ランクイン調査
#
#　return 0 : ランクインしていない
#　return 1 : ランクインした
#
#******************************************************************************#
sub checkRankIn{

	#引数取得
	my( $data, $checkList ) = @_;

	#得点の取得
	my $point = ( split( /\,/,$data ) )[0];

	#配列内の最後のデータ取得
	my $tmpCsv = @$checkList[ $DATA_MAX_NUM-1 ];

	#配列内最後のデータの得点取得
	my $tmpPoint = ( split( /\,/,$tmpCsv ) )[0];

	#得点比較
	if( $point <= $tmpPoint ){ return 0; } #ランクインしていない
	else					 { return 1; } #ランクインした
}

#******************************************************************************#
#
#　同一IDデータ調査
#
#******************************************************************************#
sub checkSameID{

	#引数取得
	my( $data, $checkList ) = @_;

	#各データ取得
	my $point = ( split( /\,/,$data ) )[0];
	my $id    = ( split( /\,/,$data ) )[1];

	#配列要素分繰り返す
	my $i = -1;
	foreach my $tmpCsv ( @$checkList ){

		#カウント
		$i++;

		#各データ取得
		my $tmpPoint = ( split( /\,/,$tmpCsv ) )[0];
		my $tmpId    = ( split( /\,/,$tmpCsv ) )[1];

		#IDが異なる場合 : コンティニュー
		if( $id != $tmpId ){ next; }

		#得点が前回より小さい場合 : 処理終了
		if( $point <= $tmpPoint ){ return; }

		#ループ脱出
		last;
	}
	#同一IDが存在しない / 存在する場合
	if( $i == $DATA_MAX_NUM-1 ){ splice( @$checkList, $DATA_MAX_NUM-1, 1 ); }
	else					   { splice( @$checkList, $i, 1				 ); }

	#リストの最後にデータ追加
	push @$checkList, $data . "\n";
}

#******************************************************************************#
#
#　自分のランク番号の取得
#
#******************************************************************************#
sub getRankNum{

	#引数取得
	my( $data, $checkList ) = @_;

	#ID取得
	my $id = ( split( /\,/,$data ) )[1];

	#配列要素分繰り返す
	my $i = 0;
	foreach my $tmpCsv ( @$checkList ){

		#順位カウント
		$i++;

		#ID取得
		my( $tmpId ) = ( split( /\,/,$tmpCsv ) )[1];

		#IDが異なる場合 : コンティニュー
		if( $id != $tmpId ){ next; }

		#順位返却
		return $i;
	}
}

